#include <math.h>

#include <unistd.h>
#include <sys/param.h>
#include <stdlib.h>

#include <iostream>

#include "FileInversion.h"
#include "../config.h"

#include "netcdf.h"

// Needed for the definition of NaN (Not a Number)
#include <limits>

extern const char* program_name_;
extern bool merlin_client_debug_;

inline double my_I_max(const MERLIN_Profile &pp)
{
    int size = pp.stokes.I.size();
    if (size < 2)
        throw string("the function my_I_max can only deal with vectors with size greater than 2");
	
    double max = pp.stokes.I[0];
	
    for (int h=1; h < size; h++)
    {
        if (pp.stokes.I[h] > max)
            max = pp.stokes.I[h];
    }	
    return max;
}

FileInversion::FileInversion():_file(0), _guess_file(0), _profile_dumper_set(false), _n_data(0), _inverter(0)
{
    //
    //string s = load_spectral_grid_name(); 
    //_par.load_regions_and_spectral_lines(s);
    //cout<<_par;
    _stokes_I = 0;
    _stokes_U = 0;
    _stokes_Q = 0;
    _stokes_V = 0;
}

void FileInversion::clean_memory()
{
    // release memory
    if (_stokes_I)
        delete [] _stokes_I; _stokes_I = 0;
    if (_stokes_Q)
        delete [] _stokes_Q; _stokes_Q = 0;
    if (_stokes_U)
        delete [] _stokes_U; _stokes_U = 0;
    if (_stokes_V)
        delete [] _stokes_V; _stokes_V = 0;
    
}

FileInversion::~FileInversion()
{
    _file=0; 
    _guess_file=0;
    if (_inverter) 
        delete _inverter;
    clean_memory();
}


float  FileInversion::get_net_polarization_of_full_spectrum()
{
    
    float I_sum = 0.0;
    float Q_sum = 0.0;
    float U_sum = 0.0;
    float V_sum = 0.0;
	
	
    int number_points = 0;
	
    for (int i=0; i < _size_of_vectors; i++) 
    {
		
        I_sum += _stokes_I[i];   
        Q_sum += _stokes_Q[i] * _stokes_Q[i];    
        U_sum += _stokes_U[i] * _stokes_U[i];     
        V_sum += _stokes_V[i] * _stokes_V[i];
        
        number_points++;
        
    }
    
    
    // normalize data
    I_sum = I_sum / number_points;
    Q_sum = Q_sum / number_points;
    U_sum = U_sum / number_points;
    V_sum = V_sum / number_points;
    
    //cout<<"NET_POL "<<( sqrt(Q_sum + U_sum + V_sum) / I_sum )<<" for ndata "<<number_points<<endl;
	
    return ( sqrt(Q_sum + U_sum + V_sum) / I_sum );
}

double FileInversion::get_net_polarization_of_constrained_spectrum()
{
    double I_sum = 0.0;
    double Q_sum = 0.0;
    double U_sum = 0.0;
    double V_sum = 0.0;
    
    
    int number_points = 0;
    
    for (int i=0; i < _n_data; i++) 
    {
		
        I_sum += observed.I[i];   
        Q_sum += observed.Q[i] * observed.Q[i];
        U_sum += observed.U[i] * observed.U[i];
        V_sum += observed.V[i] * observed.V[i];
        
        number_points++;
        
    }
    
    
    // normalize data
    I_sum = I_sum / number_points;
    Q_sum = Q_sum / number_points;
    U_sum = U_sum / number_points;
    V_sum = V_sum / number_points;
    
    //cout<<"NET_POL_CONSTRAINED_SPECTRUM "<<( sqrt(Q_sum + U_sum + V_sum) / I_sum )<<" for ndata "<<number_points<<endl;
	
    return ( sqrt(Q_sum + U_sum + V_sum) / I_sum );
}


void FileInversion::set_observed_profile_by_ranges()
{
    
    observed.I.resize(_n_data, 0.0);	
    observed.Q.resize(_n_data, 0.0);	
    observed.U.resize(_n_data, 0.0);
    observed.V.resize(_n_data, 0.0);
    observed.lambda.resize(_n_data, 0.0);
	
    int h = 0;
	
    for (int i=0; i < ranges.size(); i++) 
    {
		
        for (register int j=ranges[i].first; j <=ranges[i].last; j++)  
        {
            if (j < _size_of_vectors)
            {
                observed.I[h] = (double) _stokes_I[j];
                observed.Q[h] = (double) _stokes_Q[j];
                observed.U[h] = (double) _stokes_U[j];
                observed.V[h] = (double) _stokes_V[j];
                observed.lambda[h] = ranges[i].wave_base + ( (j - ranges[i].first) * ranges[i].wave_step ) / 1000;
                h++;
            }
            else
                throw string("Possible SEGV due to missmatched size in the Stokes vectors and the Ranges of operation");
        }
    }
	
}



void FileInversion::run(CSACFile *file, InitialGuessFile *guess, const string &outfile) throw(string, CSACFile)
{
	float *cache_stokes_I, *cache_stokes_Q, *cache_stokes_U, *cache_stokes_V;

    if (!_inverter)
        throw string("No inverter has been set for this object of type  FileInversion to compute the solutions for Ax = b");

    _inverter->set_maximun_iterations(CSAC::max_number_iterations);
    _inverter->set_max_damping_parameter(CSAC::max_damping_parameter);
    _inverter->set_min_damping_parameter(CSAC::min_damping_parameter);
    _inverter->set_max_number_failures(CSAC::max_number_failures);
    _inverter->set_worst_tolerated_condition_number(CSAC::worst_tolerated_condition_number);
    // _inverter->set_svd_threshold(CSAC::SVD_threshold);
    if (merlin_client_debug_)
        ((MERLIN_Object *) _inverter)->dump(cout);

    static int total_effort = 0;
    _file = file;
    
    if (guess)
        _guess_file = guess;
    
    
	
    
    if (_par.stray_light_profile.set())
    {
        double max_value_for_planck_function = 2.0 * _par.stray_light_profile.get_continuum();
        _inverter->set_max_planck_function(max_value_for_planck_function);
        _inverter->set_max_planck_function_gradient(max_value_for_planck_function);
    }
    
    if(merlin_client_debug_)
    {
        cout<<"Working with "<<_file->get_number_of_pixels()<<" pixels (y slit positions)"<<endl;
        cout<<"working with "<<_file->get_size_of_vectors()<<" elements per pixel (wavelength)"<<endl;
    }
    _file->get_inversion_lambda_ranges(ranges);
    if (merlin_client_debug_)
    {
        for (int t =0; t < ranges.size(); t++)
        {
            cout<<"Ranges "<<t+1<<endl;
            cout<<"      First: "<<ranges[t].first<<" Last: "<<ranges[t].last<<endl;
        }
    }
	
    for (int i=0; i < ranges.size(); i++) 
        _n_data += (ranges[i].last - ranges[i].first + 1);
	
	
    _size_of_vectors = _file->get_size_of_vectors();
    clean_memory();
    
    cache_stokes_I = new float[(Y_max-Y_min+1)*_size_of_vectors];
    cache_stokes_Q = new float[(Y_max-Y_min+1)*_size_of_vectors];
    cache_stokes_U = new float[(Y_max-Y_min+1)*_size_of_vectors];
    cache_stokes_V = new float[(Y_max-Y_min+1)*_size_of_vectors];
    _stokes_I = new float[_size_of_vectors];
    _stokes_Q = new float[_size_of_vectors];
    _stokes_U = new float[_size_of_vectors];
    _stokes_V = new float[_size_of_vectors];
	
    if ( _file->get_number_of_pixels() < Y_max)
        throw string("The y_max value is greater than the maximun number of wavelength");
	
    double netpol;
    double netpol_full;
    
    int this_effort = 0;
    
    if (merlin_client_debug_)
        cout<<__FILE__<<":"<<__LINE__
            <<": Running with a polarization threshold of "<<_minimum_polarization<<endl;
    
    for (int h=Y_min; h <= Y_max; h++) {
        _file->get_I(h, cache_stokes_I+(h-Y_min)*_size_of_vectors, _size_of_vectors);
        _file->get_Q(h, cache_stokes_Q+(h-Y_min)*_size_of_vectors, _size_of_vectors);
        _file->get_U(h, cache_stokes_U+(h-Y_min)*_size_of_vectors, _size_of_vectors);
        _file->get_V(h, cache_stokes_V+(h-Y_min)*_size_of_vectors, _size_of_vectors);
	}

    if (_profile_dumper_set) {
	    prime_netCDF_file(outfile);
	}

    for (int h=Y_min; h <= Y_max; h++)
    {
        if (!merlin_client_debug_)
            cout<<'.'<<flush;
		
		memcpy(_stokes_I, cache_stokes_I+(h-Y_min)*_size_of_vectors, _size_of_vectors * sizeof(float));
		memcpy(_stokes_Q, cache_stokes_Q+(h-Y_min)*_size_of_vectors, _size_of_vectors * sizeof(float));
		memcpy(_stokes_U, cache_stokes_U+(h-Y_min)*_size_of_vectors, _size_of_vectors * sizeof(float));
		memcpy(_stokes_V, cache_stokes_V+(h-Y_min)*_size_of_vectors, _size_of_vectors * sizeof(float));
        
        // Determine the max value of I for the observed continuum
        _Imax = _stokes_I[0];
        for (register int y=1; y < _size_of_vectors; y++)
        {
            if (_Imax < _stokes_I[y])
                _Imax = _stokes_I[y];
        }
        
        _observed_continuum.push_back((double) _Imax);
        
        _inverter->set_continuum_normalization((double)_Imax);
        
        netpol_full = (double) get_net_polarization_of_full_spectrum();
        
        set_observed_profile_by_ranges();
        
        netpol = get_net_polarization_of_constrained_spectrum();
        
        _net_polarization.push_back(netpol);
        _unconstrained_net_polarization.push_back(netpol_full);
        
        if (netpol > _minimum_polarization)
        {
            call_inversion_kernel(h, this_effort, netpol);
            total_effort += this_effort;
        }
        else
        {
            // This seems to work better than the definition from math.h FP_NAN
            //double nan = 0.0 / 0.0;
			// There is no guarantee that 0.0/0.0 is actually NAN everywhere
			// It appears to be 0 on SunOS/Opteron using the SUN compilers, -NAN some other places.
            double nan = std::numeric_limits<double>::quiet_NaN();
            _B.push_back(nan);
            _B_error.push_back(nan);
            _Inclination.push_back(nan);
            _Inclination_error.push_back(nan);
            _Azimuth.push_back(nan);
            _Azimuth_error.push_back(nan);
            _Eta0.push_back(nan);
            _Eta0_error.push_back(nan);
            _Damp.push_back(nan);
            _Damp_error.push_back(nan);
            _Dop.push_back(nan);
            _Dop_error.push_back(nan);
            _B0.push_back(nan);
            _B0_error.push_back(nan);
            _B1.push_back(nan);
            _B1_error.push_back(nan);
            _vmac.push_back(nan);
            _vmac_error.push_back(nan);
            _lines_shift1.push_back(nan);
            _lines_shift1_error.push_back(nan);
            _lines_shift2.push_back(nan);
            _lines_shift2_error.push_back(nan);
            _stray_light.push_back(nan);
            _stray_light_error.push_back(nan);
            _stray_light_shift.push_back(nan);
            _stray_light_shift_error.push_back(nan);
            _chisq.push_back(nan);
            _chisq_I.push_back(nan);
            _chisq_Q.push_back(nan);
            _chisq_U.push_back(nan);
            _chisq_V.push_back(nan);
            _signed_stokes_V.push_back(nan);
            _n_iterations.push_back(nan);
            _reason_for_convergence.push_back(MERLIN_Inversion::UNSET_INVERSION_NOT_PERFORMED);
            _continuum.push_back(nan);
            _B_init.push_back(nan);
            _Inclination_init.push_back(nan);
            _Azimuth_init.push_back(nan);
            _Eta0_init.push_back(nan);
            _Damp_init.push_back(nan);
            _Dop_init.push_back(nan);
            _B0_init.push_back(nan);
            _B1_init.push_back(nan);
            _vmac_init.push_back(nan);
            _lines_shift1_init.push_back(nan);
            _lines_shift2_init.push_back(nan);
            _stray_light_init.push_back(nan);
            _stray_light_shift_init.push_back(nan);
            if(merlin_client_debug_)
                cout<<"No inverting point: "<<h<<endl;
        }
		
    }
    if(!merlin_client_debug_)
        cout<<endl;
    
    if (!_profile_dumper_set) {
	    prime_netCDF_file(outfile);
	}
    close_netCDF_file();
	
    cout<<program_name_<<": Total effort = "<<total_effort<<" iterations"<<endl; 

    delete [] cache_stokes_I;
    delete [] cache_stokes_Q;
    delete [] cache_stokes_U;
    delete [] cache_stokes_V;
}

double FileInversion::self_initialize_pixel(double &netpol)
{
    double genetic_chisq;
    _par.set_n_total_free_parameters();
    //cout<<"Imax = "<<_Imax<<endl;
    MERLIN_Init::create_initial_guess(
                                      _stokes_I,
                                      _stokes_Q,
                                      _stokes_U,
                                      _stokes_V,
                                      _size_of_vectors,
                                      observed,
                                      _Imax,
                                      _par,
                                      genetic_chisq,
                                      netpol,
                                      individuals,
                                      generations
                                      );
    //cout<<_par<<endl;
    //exit(1);
    return genetic_chisq;
}


void FileInversion::call_inversion_kernel(int pos, int &niter, double &netpol)
{	
    if (merlin_client_debug_)
        cout<<"Calling inversion kernel"<<endl;
    
    if (_guess_file)
        _guess_file->load_guess_at_pixel(pos - Y_min, _par);
    else
        self_initialize_pixel(netpol);
    
    MERLIN_Model my_init;
    my_init = _par;
    
    if (merlin_client_debug_ || CSAC::merlin_debug)
    {
        cout<<"\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Index Number "<<pos - Y_min + 1<<" %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"<<endl;
        if(! CSAC::merlin_debug)
        {
            cout<<"INITIAL_GUESS: "<<endl;
            cout<<_par;
        }
    }
    
    int n_iterations;
    
    double chisq_I;
    double chisq_Q;
    double chisq_U;
    double chisq_V;
    
    double chisq = _inverter->execute(observed, _par, 0, n_iterations, chisq_I, chisq_Q, chisq_U, chisq_V);
    
    // set this value so the caller off this method can totalize the effort
    niter = n_iterations;
    
    // save the solution for this pixel
    _B.push_back(_par.magnetic_field.value);
    _B_error.push_back(_par.magnetic_field.error);
    _Inclination.push_back(_par.inclination.value * 180. / CSAC::Pi);
    _Inclination_error.push_back(_par.inclination.error* 180. / CSAC::Pi); // SAG
    _Azimuth.push_back(_par.azimuth.value * 180. / CSAC::Pi);
    _Azimuth_error.push_back(_par.azimuth.error* 180. / CSAC::Pi); // SAG
    _Eta0.push_back(_par.line_strength.value);
    _Eta0_error.push_back(_par.line_strength.error);
    _Damp.push_back(_par.damping.value);
    _Damp_error.push_back(_par.damping.error);
    _Dop.push_back(_par.doppler_width.value);
    _Dop_error.push_back(_par.doppler_width.error);
    _B0.push_back(_par.planck_function.value);
    _B0_error.push_back(_par.planck_function.error);
    _B1.push_back(_par.planck_function_gradient.value);
    _B1_error.push_back(_par.planck_function_gradient.error);
    _vmac.push_back(_par.vmac.value);
    _vmac_error.push_back(_par.vmac.error);
    _lines_shift1.push_back(_par.lines[0].wavelength_shift.value);
    _lines_shift1_error.push_back(_par.lines[0].wavelength_shift.error);
    _lines_shift2.push_back(_par.lines[1].wavelength_shift.value);
    _lines_shift2_error.push_back(_par.lines[1].wavelength_shift.error);
    _stray_light.push_back(_par.stray_light.value);
    _stray_light_error.push_back(_par.stray_light.error);
    _stray_light_shift.push_back(_par.stray_light_shift.value);
    _stray_light_shift_error.push_back(_par.stray_light_shift.error);
    _chisq.push_back(chisq);
    _chisq_I.push_back(chisq_I);
    _chisq_Q.push_back(chisq_Q);
    _chisq_U.push_back(chisq_U);
    _chisq_V.push_back(chisq_V);
    _signed_stokes_V.push_back(compute_magnitude_stokes_V());
    _n_iterations.push_back(n_iterations);
    _reason_for_convergence.push_back((double) _inverter->get_exit_condition());
    
    // save which initialization was used
    _B_init.push_back(my_init.magnetic_field.value);
    _Inclination_init.push_back(my_init.inclination.value * 180. / CSAC::Pi);
    _Azimuth_init.push_back(my_init.azimuth.value * 180. / CSAC::Pi);
    _Eta0_init.push_back(my_init.line_strength.value);
    _Damp_init.push_back(my_init.damping.value);
    _Dop_init.push_back(my_init.doppler_width.value);
    _B0_init.push_back(my_init.planck_function.value);
    _B1_init.push_back(my_init.planck_function_gradient.value);
    _vmac_init.push_back(my_init.vmac.value);
    _lines_shift1_init.push_back(my_init.lines[0].wavelength_shift.value);
    _lines_shift2_init.push_back(my_init.lines[1].wavelength_shift.value);
    _stray_light_init.push_back(my_init.stray_light.value);
    _stray_light_shift_init.push_back(my_init.stray_light_shift.value);  
    
    
    MERLIN_MilneEddington my_f;
    MERLIN_Profile my_p;
    my_f.get_profile(_par,my_p);
    if (_profile_dumper_set)
        write_profiles_to_dumper(observed, my_p.stokes, pos - Y_min);
    _continuum.push_back( my_I_max(my_p) );
    
    if (merlin_client_debug_)
    {
        cout<<"=================================================================================================="<<endl;
        cout<<"Pos: "<<pos<<", Chisq: "<<chisq<<", N iterations: "<<n_iterations<<", N free parameters: "<<_par.n_total_free_parameters<<endl;
        cout<<"=================================================================================================="<<endl;
        cout<<"OBSERVED PROFILE:"<<endl;
        observed.print_stokes_vectors(cout);
        cout<<"=================================================================================================="<<endl;
        cout<<"FITTED PROFILE:"<<endl;
        my_p.print_stokes_vectors(cout,"s_lambda","s_I","s_Q","s_U","s_V");
        cout<<"=================================================================================================="<<endl;
        cout<<"SOLUTION: "<<endl;
        cout<<_par;
    }
    
}

//string
//FileInversion::load_spectral_grid_name()
//{
//  char buf[MAXPATHLEN+1];
//  getcwd(buf, MAXPATHLEN+1);
//  if (merlin_client_debug_)
//    cout<<"Current working directory: "<<buf<<endl;
//	
//  char* spectral_grid=getenv("CSAC_SPECTRAL_GRID"); 
//  if (!spectral_grid)
//    {
//      throw string("Please set the environment variable CSAC_SPECTRAL_GRID to the location of the test file");
//    }
//	
//  string test = string(spectral_grid);
//  if (merlin_client_debug_)
//    cout<<"Loading grid from file "<<test<<endl;
//  return test;
//}
    
void
FileInversion::prime_netCDF_file(const string &outfile)
{
    ncopts = 0;
    int status = 0;
	
    
    // Metadata variables to be added in the netCDF file
    string long_name, units, range, comment;
    
    status = nc_create(outfile.c_str(), NC_CLOBBER, &_out_ncid);
    if (status != NC_NOERR) 
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_open",status,"");
        }
        string error = string(": can not open file ") + outfile;
        throw error;
    }
    
    // Global data
    string global_data("Inversion results for ");
    global_data += _file->get_file_name();
    nc_put_att_text(_out_ncid, NC_GLOBAL, "Reference", global_data.size(), global_data.c_str());
    global_data = "Computed with MERLIN version " + string(PACKAGE_VERSION);
    //cout<<"Global "<<global_data<<endl;
    nc_put_att_text(_out_ncid, NC_GLOBAL, "Version_Control", global_data.size(), global_data.c_str());
    char hostname [256];
    // Only add this data if we can get the hostname
    // if not, then ignore
    if (gethostname(hostname, 256) == 0)
    {
        global_data = "Processed by: ";
        global_data += hostname;
        nc_put_att_text(_out_ncid, NC_GLOBAL, "Host_Machine", global_data.size(), global_data.c_str());
    }
    else
        cerr<<"Notice: Could not get hostname"<<endl;
         
    // Obtain how many pixels along the Y dimension we are inverting 
    int n_pixels = Y_max - Y_min +1;
    
    if (merlin_client_debug_)
        cout<<"Inverting a total of "<<n_pixels<<" pixels"<<endl;
	
    // Add the one and only dimension for all variables
    if (nc_def_dim(_out_ncid,"len",n_pixels,&_dim_ids[0]) != NC_NOERR)
        throw string("Can not create the dimension for the output arrays");
	
    // add the vector for the net polarization
    _net_polarization.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"NET_POL",NC_DOUBLE,1,_dim_ids,&_net_polarization_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable NET_POL");
    }
    long_name = "Polarization Fraction in the defined regions";
    units = "Adimensional";
    range = "Quantity greater or equal to 0";
    comment = "Net Polarization computed only with the elements along the wavelength dimension that fell inside the regions defined in the input file.";
    nc_put_att_text(_out_ncid, _net_polarization_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _net_polarization_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _net_polarization_id, "Range", range.size(), range.c_str());
    nc_put_att_text(_out_ncid, _net_polarization_id, "Comment", comment.size(), comment.c_str());
    
    // add the vector for the unconstrained net polarization
    _unconstrained_net_polarization.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"NET_POL_FULL",NC_DOUBLE,1,_dim_ids,&_unconstrained_net_polarization_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable NET_POL_FULL");
    }
    long_name = "Polarization Fraction including all elements";
    nc_put_att_text(_out_ncid, _unconstrained_net_polarization_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _unconstrained_net_polarization_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _unconstrained_net_polarization_id, "Range", range.size(), range.c_str());
    comment = "Net Polarization computed using all the elements of stokes_I, stokes_Q, stokes_U and stokes _V ";
    nc_put_att_text(_out_ncid, _unconstrained_net_polarization_id, "Comment", comment.size(), comment.c_str());
    
    
    // add magnetic field strength
    _B.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SFLD",NC_DOUBLE,1,_dim_ids,&_B_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SFLD");
    }
    long_name = "Field Strength";
    units = "Gauss";
    range = "Range: 0 to 5000";
    nc_put_att_text(_out_ncid, _B_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _B_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _B_id, "Range", range.size(), range.c_str());
    
    // add magnetic field strength error
    _B_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SFLD_ERROR",NC_DOUBLE,1,_dim_ids,&_B_error_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SFLD_ERROR");
    }
    long_name = "Error of Field Strength";
    nc_put_att_text(_out_ncid, _B_error_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // add magnetic field inclination
    _Inclination.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SPSI",NC_DOUBLE,1,_dim_ids,&_Inclination_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SPSI");
    }
    long_name = "Field Inclination";
    units = "Degrees";
    range = "Range: 0 to 180 degrees";
    nc_put_att_text(_out_ncid, _Inclination_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _Inclination_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _Inclination_id, "Range", range.size(), range.c_str());
    
    // add magnetic field inclination Error
    _Inclination_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SPSI_ERROR",NC_DOUBLE,1,_dim_ids,&_Inclination_error_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SPSI_ERROR");
    }
    long_name = "Error of Field Inclination";
    nc_put_att_text(_out_ncid, _Inclination_error_id, "Long_Name", long_name.size(), long_name.c_str());
    
    
    // add magnetic field azimuth
    _Azimuth.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SAZM",NC_DOUBLE,1,_dim_ids,&_Azimuth_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SAZM");
    }
    long_name = "Field Azimuth";
    units = "Degrees";
    range = "Range: 0 to 180 degrees";
    nc_put_att_text(_out_ncid, _Azimuth_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _Azimuth_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _Azimuth_id, "Range", range.size(), range.c_str());
    
    // add magnetic field azimuth Error
    _Azimuth_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SAZM_ERROR",NC_DOUBLE,1,_dim_ids,&_Azimuth_error_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SAZM_ERROR");
    }
    long_name = "Error for Field Azimuth";
    nc_put_att_text(_out_ncid, _Azimuth_error_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // add line strength
    _Eta0.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SETA0",NC_DOUBLE,1,_dim_ids,&_Eta0_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SETA0");
    }
    long_name = "Line Strength";
    units = "Dimensionless";
    range = "1E-2 to 100";
    nc_put_att_text(_out_ncid, _Eta0_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _Eta0_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _Eta0_id, "Range", range.size(), range.c_str());
    
    // add line strength Error
    _Eta0_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SETA0_ERROR",NC_DOUBLE,1,_dim_ids,&_Eta0_error_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SETA0_ERROR");
    }
    long_name = "Error for Line Strength";
    nc_put_att_text(_out_ncid, _Eta0_error_id, "Long_Name", long_name.size(), long_name.c_str());
    
	
    // add damping
    _Damp.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SDMP",NC_DOUBLE,1,_dim_ids,&_Damp_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SDMP");
    }
    long_name = "Damping Parameter, Relative to Doppler Width";
    units = "Dimesionless";
    range = "0 to 1.5";
    nc_put_att_text(_out_ncid, _Damp_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _Damp_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _Damp_id, "Range", range.size(), range.c_str());
    
    // add damping Error
    _Damp_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SDMP_ERROR",NC_DOUBLE,1,_dim_ids,&_Damp_error_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SDMP_ERROR");
    }
    long_name = "Error for the Damping";
    nc_put_att_text(_out_ncid, _Damp_error_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // add doppler width
    _Dop.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SDOP",NC_DOUBLE,1,_dim_ids,&_Dop_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SDOP");
    }
    long_name = "Doppler Width";
    units = "mA";
    range = "20 to 90";
    nc_put_att_text(_out_ncid, _Dop_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _Dop_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _Dop_id, "Range", range.size(), range.c_str());
    
    // add doppler width error
    _Dop_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SDOP_ERROR",NC_DOUBLE,1,_dim_ids,&_Dop_error_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SDOP_ERROR");
    }
    long_name = "Error for the Doppler Width";
    nc_put_att_text(_out_ncid, _Dop_error_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // add B0
    _B0.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SBZERO",NC_DOUBLE,1,_dim_ids,&_B0_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SBZERO");
    }
    long_name = "Source Function";
    units = "Same Units as the Intensity";
    range = "0 to twice the continuum";
    nc_put_att_text(_out_ncid, _B0_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _B0_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _B0_id, "Range", range.size(), range.c_str());
    
    // add B0 error
    _B0_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SBZERO_ERROR",NC_DOUBLE,1,_dim_ids,&_B0_error_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SBZERO_ERROR");
    }
    long_name = "Error for the Source Function";
    nc_put_att_text(_out_ncid, _B0_error_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // add B1
    _B1.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SB1MU",NC_DOUBLE,1,_dim_ids,&_B1_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SB1MU");
    }
    long_name = "Source Function Gradient";
    units = "Same Units as the Intensity";
    range = "0 to twice the continuum";
    nc_put_att_text(_out_ncid, _B1_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _B1_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _B1_id, "Range", range.size(), range.c_str());
    
    // add B1 error
    _B1_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SB1MU_ERROR",NC_DOUBLE,1,_dim_ids,&_B1_error_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SB1MU_ERROR");
    }
    long_name = "Error for the Source Function Gradient";
    nc_put_att_text(_out_ncid, _B1_error_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // add vmac
    _vmac.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"VMAC",NC_DOUBLE,1,_dim_ids,&_vmac_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable VMAC");
    }
    long_name = "Macroturbulence";
    units = "Km/sec";
    range = "0 to 3.0";
    nc_put_att_text(_out_ncid, _vmac_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _vmac_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _vmac_id, "Range", range.size(), range.c_str());
    
    // add vmac error
    _vmac_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"VMAC_ERROR",NC_DOUBLE,1,_dim_ids,&_vmac_error_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable VMAC_ERROR");
    }
    long_name = "Error for the Macroturbulence";
    nc_put_att_text(_out_ncid, _vmac_error_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // add the spectral lines to the netCDF file.
    // only two lines are allowed
    int n_lines_model = _par.lines.size();
    if (n_lines_model != 2)
    {
        char error_line[32];
        snprintf(error_line,sizeof(error_line), "%i", __LINE__);
        char my_lines[32];
        snprintf(my_lines, sizeof(my_lines), "%i",n_lines_model); 
        string error=__FILE__ + string(":") + string(error_line) + string(": This client can only add two lines to the output netCDF file. Please check the model defined in the spectral grid file, the model is defining ") + string(my_lines) + string(" lines");
        throw error;
    }
	
    // variable for line 6301.5091
    _lines_shift1.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SCEN2",NC_DOUBLE,1,_dim_ids,&_lines_shift1_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SCEN2");
    }
    long_name = "Doppler Shift for the center of the line 6301.5091";
    units = "Km/sec";
    range = "-10.0 to 10.0";
    nc_put_att_text(_out_ncid, _lines_shift1_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _lines_shift1_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _lines_shift1_id, "Range", range.size(), range.c_str());
    
    // variable for line 6301.5091 error
    _lines_shift1_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SCEN2_ERROR",NC_DOUBLE,1,_dim_ids,&_lines_shift1_error_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SCEN2_ERROR");
    }
    long_name = "Error for the Doppler Shift for the center of the line 6301.5091";
    nc_put_att_text(_out_ncid, _lines_shift1_error_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // variable for line 6302.5017
    _lines_shift2.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SCEN1",NC_DOUBLE,1,_dim_ids,&_lines_shift2_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SCEN1");
    }	
    long_name = "Doppler Shift for the center of the line 6302.5017";
    units = "Km/sec";
    range = "-10.0 to 10.0";
    nc_put_att_text(_out_ncid, _lines_shift2_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _lines_shift2_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _lines_shift2_id, "Range", range.size(), range.c_str());
    
    // variable for line 6302.5017
    _lines_shift2_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SCEN1_ERROR",NC_DOUBLE,1,_dim_ids,&_lines_shift2_error_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SCEN1_ERROR");
    }	
    long_name = "Error for the Doppler Shift for the center of the line 6302.5017";
    nc_put_att_text(_out_ncid, _lines_shift2_error_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // Variable for stray_light_fraction		
    _stray_light.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SALPHA",NC_DOUBLE,1,_dim_ids,&_stray_light_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SALPHA");
    }
    if (CSAC::stray_light_model == CSAC::fill)
        long_name = "Stray Light Fill Factor (Notice, this is not the Stray Light Fraction)";
    else if (CSAC::stray_light_model == CSAC::fraction)
        long_name = "Stray Light Fraction (Notice, this is not the Stray Light Fill Factor)";
    else
    {
        char error_line[32];
        sprintf(error_line, "%i", __LINE__);
        string t_error=__FILE__ + string(":") + string(error_line) + ": the stray_light_model is invalid\n";
        throw t_error;
    }
    units = "------";
    range = "0 to 1.0";
    nc_put_att_text(_out_ncid, _stray_light_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _stray_light_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _stray_light_id, "Range", range.size(), range.c_str());
    
    
    // Variable for stray_light_fraction error		
    _stray_light_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SALPHA_ERROR",NC_DOUBLE,1,_dim_ids,&_stray_light_error_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SALPHA_ERROR");
    }
    long_name = "Error for the Stray Light Fraction";
    nc_put_att_text(_out_ncid, _stray_light_error_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // Variable for stray light shift		
    _stray_light_shift.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SDELTA",NC_DOUBLE,1,_dim_ids,&_stray_light_shift_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SDELTA");
    }
    long_name = "Doppler Shift for the center of the Stray Light vector";
    units = "Km/sec";
    range = "-10.0 to 10.0";
    nc_put_att_text(_out_ncid, _stray_light_shift_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _stray_light_shift_id, "Units", units.size(), units.c_str());
    nc_put_att_text(_out_ncid, _stray_light_shift_id, "Range", range.size(), range.c_str());
    
    // Variable for stray light shift error	
    _stray_light_shift_error.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SDELTA_ERROR",NC_DOUBLE,1,_dim_ids,&_stray_light_shift_error_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SDELTA_ERROR");
    }
    long_name = "Error for the Doppler Shift for the center of the Stray Light vector";
    nc_put_att_text(_out_ncid, _stray_light_shift_error_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // Variable for chisq		
    _chisq.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"CHISQ",NC_DOUBLE,1,_dim_ids,&_chisq_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable CHISQ");
    }
    long_name = "Chisq for the combined Stokes I, Q, U and V. This is normalize to the square of the continuum.";
    nc_put_att_text(_out_ncid, _chisq_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // Variable for chisq_I		
    _chisq_I.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"CHISQ_I",NC_DOUBLE,1,_dim_ids,&_chisq_I_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable CHISQ_I");
    }
    long_name = "Chisq for the combined Stokes I. This is normalize to the square of the continuum.";
    nc_put_att_text(_out_ncid, _chisq_I_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // Variable for chisq_Q		
    _chisq_Q.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"CHISQ_Q",NC_DOUBLE,1,_dim_ids,&_chisq_Q_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable CHISQ_Q");
    }
    long_name = "Chisq for the combined Stokes Q. This is normalize to the square of the continuum.";
    nc_put_att_text(_out_ncid, _chisq_Q_id, "Long_Name", long_name.size(), long_name.c_str());
    
    
    // Variable for chisq_U		
    _chisq_U.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"CHISQ_U",NC_DOUBLE,1,_dim_ids,&_chisq_U_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable CHISQ_U");
    }
    long_name = "Chisq for the combined Stokes U. This is normalize to the square of the continuum.";
    nc_put_att_text(_out_ncid, _chisq_U_id, "Long_Name", long_name.size(), long_name.c_str());
    
    
    // Variable for chisq		
    _chisq_V.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"CHISQ_V",NC_DOUBLE,1,_dim_ids,&_chisq_V_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable CHISQ_V");
    }
    long_name = "Chisq for the combined Stokes V. This is normalize to the square of the continuum.";
    nc_put_att_text(_out_ncid, _chisq_V_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // Variable for signed Stokes V
    _signed_stokes_V.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SIGNED_V",NC_DOUBLE,1,_dim_ids,&_signed_stokes_V_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SIGNED_V");
    }
    long_name = "Magnitude of stokes V profile";
    nc_put_att_text(_out_ncid, _signed_stokes_V_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // Variable for number of iterations
    _n_iterations.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"NITER",NC_DOUBLE,1,_dim_ids,&_n_iterations_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable NITER");
    }
    
    long_name = "Number of Iterations performed";
    range = " 0 to the MAX set for this inversion. Please refer to your input XML File";
    nc_put_att_text(_out_ncid, _n_iterations_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _n_iterations_id, "Range", range.size(), range.c_str());
	
    // Variable for the theoretiical continuum
    _continuum.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"SCCT",NC_DOUBLE,1,_dim_ids,&_continuum_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable SCCT");
    }
    long_name = "Theoretical Continuum based on Stokes I for the synthetic signal computed at the best fit";
    nc_put_att_text(_out_ncid, _continuum_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // Variable for the observed continuum
    _observed_continuum.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"OSCCT",NC_DOUBLE,1,_dim_ids,&_observed_continuum_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable OSCCT");
    }
    long_name = "Observed continuum, defined as the max for Stokes I in the observed signal.";
    nc_put_att_text(_out_ncid, _observed_continuum_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // Variable for the reason for convergence
    _reason_for_convergence.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"RCONV",NC_DOUBLE,1,_dim_ids,&_rconv_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable RCONV");
    }
    long_name = "Reason of the  Levenberg-Marquardt Algorithm to return";
    range = "Possible Values: UNSET_INVERSION_NOT_PERFORMED = 0, REACHED_MAX_ITERATIONS = 1, REACHED_MAX_FAILURES = 2, REACH_CONVERGENCE_CRITERIA = 3, LAMBDA_GREATER_THAM_MAX_ALLOWED = 4, LAMBDA_SMALLER_THAM_MIN_ALLOWED = 5, NOTHING_TO_DO = 6. Please refer to the documentaion about the inversion algorithm for more about the meaning of each of these constants.";
    nc_put_att_text(_out_ncid, _rconv_id, "Long_Name", long_name.size(), long_name.c_str());
    nc_put_att_text(_out_ncid, _rconv_id, "Range", range.size(), range.c_str());
    
    // add magnetic field strength init
    _B_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SFLD",NC_DOUBLE,1,_dim_ids,&_B_init_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SFLD");
    }
    long_name = "Initialization Value for SFLD";
    nc_put_att_text(_out_ncid, _B_init_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // add magnetic field inclination init
    _Inclination_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SPSI",NC_DOUBLE,1,_dim_ids,&_Inclination_init_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SPSI");
    }
    long_name = "Initialization Value for SPSI";
    nc_put_att_text(_out_ncid, _Inclination_init_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // add magnetic field azimuth init
    _Azimuth_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SAZM",NC_DOUBLE,1,_dim_ids,&_Azimuth_init_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SAZM");
    }
    long_name = "Initialization Value for SAZM";
    nc_put_att_text(_out_ncid, _Azimuth_init_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // add line strength init
    _Eta0_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SETA0",NC_DOUBLE,1,_dim_ids,&_Eta0_init_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SETA0");
    }
    long_name = "Initialization Value for SETA0";
    nc_put_att_text(_out_ncid, _Eta0_init_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // add damping init
    _Damp_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SDMP",NC_DOUBLE,1,_dim_ids,&_Damp_init_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SDMP");
    }
    long_name = "Initialization Value for SDMP";
    nc_put_att_text(_out_ncid, _Damp_init_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // add doppler width init
    _Dop_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SDOP",NC_DOUBLE,1,_dim_ids,&_Dop_init_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SDOP");
    }
    long_name = "Initialization Value for SDOP";
    nc_put_att_text(_out_ncid, _Dop_init_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // add B0 init
    _B0_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SBZERO",NC_DOUBLE,1,_dim_ids,&_B0_init_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SBZERO");
    }
    long_name = "Initialization Value for SBZERO"; 
    nc_put_att_text(_out_ncid, _B0_init_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // add B1 init
    _B1_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SB1MU",NC_DOUBLE,1,_dim_ids,&_B1_init_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SB1MU");
    }
    long_name = "Initialization Value for SB1MU";
    nc_put_att_text(_out_ncid, _B1_init_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // add vmac init
    _vmac_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_VMAC",NC_DOUBLE,1,_dim_ids,&_vmac_init_id);
    if (status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_VMAC");
    }
    long_name = "Initialization Value for VMAC";
    nc_put_att_text(_out_ncid, _vmac_init_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // variable for line 6301.5 init
    _lines_shift1_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SCEN2",NC_DOUBLE,1,_dim_ids,&_lines_shift1_init_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SCEN2");
    }
    long_name = "Initialization Value for SCEN2";
    nc_put_att_text(_out_ncid, _lines_shift1_init_id, "Long_Name", long_name.size(), long_name.c_str());
    
    // variable for line 6302.5 init
    _lines_shift2_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SCEN1",NC_DOUBLE,1,_dim_ids,&_lines_shift2_init_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SCEN1");
    }
    long_name = "Initialization Value for SCEN1";
    nc_put_att_text(_out_ncid, _lines_shift2_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // Variable for stray_light_fraction init	
    _stray_light_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SALPHA",NC_DOUBLE,1,_dim_ids,&_stray_light_init_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SALPHA");
    }
    long_name = "Initialization Value for SALPHA";
    nc_put_att_text(_out_ncid, _stray_light_init_id, "Long_Name", long_name.size(), long_name.c_str());
	
    // Variable for stray light shift		
    _stray_light_shift_init.reserve(n_pixels);
    status = nc_def_var(_out_ncid,"INIT_SDELTA",NC_DOUBLE,1,_dim_ids,&_stray_light_shift_init_id);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable INIT_SDELTA");
    }
    long_name = "Initialization Value for SDELTA";
    nc_put_att_text(_out_ncid, _stray_light_shift_init_id, "Long_Name", long_name.size(), long_name.c_str());
    
    
    // If the profiles were requested, add them to the file.
    if (_profile_dumper_set)
    {
        int profiles_dim_ids[3];
        // Add now another dimension to identify I, Q, U and V
        status = nc_def_dim(_out_ncid,"stokes",4,&profiles_dim_ids[0]); 
        if(status != NC_NOERR)
        {
            if(merlin_client_debug_)
            {
                ncopts = NC_VERBOSE;
                nc_advise("nc_def_dim",status,"");
            }
            throw string("Can not create the dimension to identify I, Q, U and V");
        }
        profiles_dim_ids[1]=_dim_ids[0];
        // Add now another dimension for the profiles length
        if (merlin_client_debug_)
            cout<<"Defining dimension for lambda with size "<<_n_data<<endl;
        
        status = nc_def_dim(_out_ncid,"lambda",_n_data,&profiles_dim_ids[2]); 
        if(status != NC_NOERR)
        {
            if(merlin_client_debug_)
            {
                ncopts = NC_VERBOSE;
                nc_advise("nc_def_dim",status,"");
            }
            throw string("Can not create the dimension for the length of the profiles");
        }
		
        status = nc_def_var(_out_ncid,"OBS",NC_DOUBLE,3,profiles_dim_ids,&_observed_id);
        if(status != NC_NOERR)
        {
            if(merlin_client_debug_)
            {
                ncopts = NC_VERBOSE;
                nc_advise("nc_def_var",status,"");
            }
            throw string("can not create variable OBS");
        }
		
        status = nc_def_var(_out_ncid,"SYN",NC_DOUBLE,3,profiles_dim_ids,&_synthetic_id);
        if(status != NC_NOERR)
        {
            if(merlin_client_debug_)
            {
                ncopts = NC_VERBOSE;
                nc_advise("nc_def_var",status,"");
            }
            throw string("can not create variable SYN");
        }
		
        status = nc_def_var(_out_ncid,"lambda",NC_DOUBLE,1,&profiles_dim_ids[2],&_lambda_id);
        if(status != NC_NOERR)
        {
            if(merlin_client_debug_)
            {
                ncopts = NC_VERBOSE;
                nc_advise("nc_def_var",status,"");
            }
            throw string("can not create variable lamda");
        }
    }
    
    status = nc_def_var(_out_ncid,"STOKES_I_WEIGHT",NC_DOUBLE, 0, 0,&_stokes_I_id);
	if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable STOKES_I_WEIGTH");
    }
    status = nc_def_var(_out_ncid,"STOKES_Q_WEIGHT",NC_DOUBLE, 0, 0,&_stokes_Q_id);
	if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable STOKES_I_WEIGTH");
    }

    status = nc_def_var(_out_ncid,"STOKES_U_WEIGHT",NC_DOUBLE, 0, 0,&_stokes_U_id);
	if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable STOKES_I_WEIGTH");
    }
    status = nc_def_var(_out_ncid,"STOKES_V_WEIGHT",NC_DOUBLE, 0, 0,&_stokes_V_id);
	if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_def_var",status,"");
        }
        throw string("can not create variable STOKES_I_WEIGTH");
    }
    
    
    // end the define mode for this netCDF file
    if (nc_enddef(_out_ncid)  != NC_NOERR)
        throw string("Can not end the definition mode for the netCDF output file"); 
    if(merlin_client_debug_)
        cout<<"Completed prime of netcdf file "<<outfile<<endl;
    
}





void
FileInversion::close_netCDF_file()
{
    int status;
	
    status = nc_put_var_double(_out_ncid, _net_polarization_id, &_net_polarization[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Net Polarization");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    
    status = nc_put_var_double(_out_ncid, _unconstrained_net_polarization_id, &_unconstrained_net_polarization[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Unconstrained Net Polarization");
    
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }

    status = nc_put_var_double(_out_ncid, _B_id, &_B[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable B");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _B_error_id, &_B_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable B error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Inclination_id, &_Inclination[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Inclination");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _Inclination_error_id, &_Inclination_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Inclination error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Azimuth_id, &_Azimuth[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Azimuth");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _Azimuth_error_id, &_Azimuth_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Azimuth error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Eta0_id, &_Eta0[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable ETA0");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _Eta0_error_id, &_Eta0_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable ETA0 error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Damp_id, &_Damp[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Damping");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _Damp_error_id, &_Damp_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Damping error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Dop_id, &_Dop[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Doppler Width");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _Dop_error_id, &_Dop_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Doppler Width error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _B0_id, &_B0[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable B0");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _B0_error_id, &_B0_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable B0 error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _B1_id, &_B1[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable B1");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _B1_error_id, &_B1_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable B1 error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _vmac_id, &_vmac[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable VMAC");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _vmac_error_id, &_vmac_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable VMAC error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    
    status = nc_put_var_double(_out_ncid, _lines_shift1_id, &_lines_shift1[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SCEN2");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _lines_shift1_error_id, &_lines_shift1_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SCEN2 error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _lines_shift2_id, &_lines_shift2[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SCEN1");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _lines_shift2_error_id, &_lines_shift2_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SCEN1 error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _stray_light_id, &_stray_light[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SALPHA");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _stray_light_error_id, &_stray_light_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SALPHA error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _stray_light_shift_id, &_stray_light_shift[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SDELTA");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _stray_light_shift_error_id, &_stray_light_shift_error[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SDELTA error");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _chisq_id, &_chisq[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable CHISQ");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }

    status = nc_put_var_double(_out_ncid, _chisq_I_id, &_chisq_I[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable CHISQ_I");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }

    status = nc_put_var_double(_out_ncid, _chisq_Q_id, &_chisq_Q[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable CHISQ_Q");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }

    status = nc_put_var_double(_out_ncid, _chisq_U_id, &_chisq_U[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable CHISQ_U");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }

    status = nc_put_var_double(_out_ncid, _chisq_V_id, &_chisq_V[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable CHISQ_V");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _signed_stokes_V_id, &_signed_stokes_V[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SIGNED_V");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _n_iterations_id, &_n_iterations[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable NITER");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _continuum_id, &_continuum[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SCCT");
    }	
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }

    status = nc_put_var_double(_out_ncid, _observed_continuum_id, &_observed_continuum[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable OSCCT");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _rconv_id, &_reason_for_convergence[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable RCONV");
    }	
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _B_init_id, &_B_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable B Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Inclination_init_id, &_Inclination_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Inclination Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Azimuth_init_id, &_Azimuth_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Azimuth Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Eta0_init_id, &_Eta0_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable ETA0 Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Damp_init_id, &_Damp_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Damping Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _Dop_init_id, &_Dop_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Doppler Width Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _B0_init_id, &_B0_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable B0 Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _B1_init_id, &_B1_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable B1 Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _vmac_init_id, &_vmac_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable VMAC Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
    
    status = nc_put_var_double(_out_ncid, _lines_shift1_init_id, &_lines_shift1_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SCEN2 Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _lines_shift2_init_id, &_lines_shift2_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable SCEN1 Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _stray_light_init_id, &_stray_light_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Stary Light Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    status = nc_put_var_double(_out_ncid, _stray_light_shift_init_id, &_stray_light_shift_init[0]);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable Stary Light Shift Init");
    }
    status = nc_sync(_out_ncid);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_sync",status,"");
        }
        throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
    }
	
    if (_profile_dumper_set)
    {
        status = nc_put_var_double(_out_ncid, _lambda_id, &observed.lambda[0]);
        if(status != NC_NOERR)
        {
            if(merlin_client_debug_)
            {
                ncopts = NC_VERBOSE;
                nc_advise("nc_put_var_double",status,"");
            }
            throw string("can not write variable lamda");
        }
        status = nc_sync(_out_ncid);
        if(status != NC_NOERR)
        {
            if(merlin_client_debug_)
            {
                ncopts = NC_VERBOSE;
                nc_advise("nc_sync",status,"");
            }
            throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
        }

    }
    
    status = nc_put_var_double(_out_ncid, _stokes_I_id, &CSAC::STOKES_I_WEIGHT);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable STOKES_I_WEIGHT");
    }
    status = nc_put_var_double(_out_ncid, _stokes_Q_id, &CSAC::STOKES_Q_WEIGHT);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable STOKES_Q_WEIGHT");
    }
    status = nc_put_var_double(_out_ncid, _stokes_U_id, &CSAC::STOKES_U_WEIGHT);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable STOKES_U_WEIGHT");
    }
    status = nc_put_var_double(_out_ncid, _stokes_V_id, &CSAC::STOKES_V_WEIGHT);
    if(status != NC_NOERR)
    {
        if(merlin_client_debug_)
        {
            ncopts = NC_VERBOSE;
            nc_advise("nc_put_var_double",status,"");
        }
        throw string("can not write variable STOKES_V_WEIGHT");
    }
	
    nc_close(_out_ncid);
}	

void 
FileInversion::write_profiles_to_dumper(const MERLIN_StokesProfile &obs, const MERLIN_StokesProfile &syn, int pos)
{
    int status;
    size_t count[3] = {1, 1, _n_data};
    size_t start[3];
    for (int stokes = 0; stokes < 4; stokes++)
    {
        switch (stokes)
        {
            case 0:
                start[0]=0;
                start[1]=pos;
                start[2]=0;
                status = nc_put_vara_double(_out_ncid, _observed_id, start, count, &obs.I[0]);
                status = nc_put_vara_double(_out_ncid, _synthetic_id, start, count, &syn.I[0]);
                break;
            case 1:
                start[0]=1;
                start[1]=pos;
                start[2]=0;
                status = nc_put_vara_double(_out_ncid, _observed_id, start, count, &obs.Q[0]);
                status = nc_put_vara_double(_out_ncid, _synthetic_id, start, count, &syn.Q[0]);
                break;
            case 2:
                start[0]=2;
                start[1]=pos;
                start[2]=0;
                status = nc_put_vara_double(_out_ncid, _observed_id, start, count, &obs.U[0]);
                status = nc_put_vara_double(_out_ncid, _synthetic_id, start, count, &syn.U[0]);
                break;
            case 3:
                start[0]=3;
                start[1]=pos;
                start[2]=0;
                status = nc_put_vara_double(_out_ncid, _observed_id, start, count, &obs.V[0]);
                status = nc_put_vara_double(_out_ncid, _synthetic_id, start, count, &syn.V[0]);
                break;
        }
        if(status != NC_NOERR)
        {
            if(merlin_client_debug_)
            {
                ncopts = NC_VERBOSE;
                nc_advise("nc_put_var_double",status,"");
            }
            char error_line[32];
            sprintf(error_line, "%i", __LINE__);
            string t_error=__FILE__ + string(":") + string(error_line) + string("STOP! Exception trapped.\n");
            t_error += string("You have requested to get the profiles in your output.");
            t_error += string("the netCDF API reports that can not write either the observed profile or the fitted profiles to the output netCDF file.");
            throw t_error;
        }
        status = nc_sync(_out_ncid);
        if(status != NC_NOERR)
        {
            if(merlin_client_debug_)
            {
                ncopts = NC_VERBOSE;
                nc_advise("nc_sync",status,"");
            }
            throw string("Failed to syncronize contents of memory with netCDF file in disk. Data loss!");
        }
    }
	
}


double FileInversion::compute_magnitude_stokes_V()
{
    // This quantity is how many pixels correspond to one Km/s 
    // of doppler shift of the line center for 6301.5091
    // (1/ 299792.458 * (6301.5091 * 1000)) / 21.549 = 9.754314266668623e-01
    static const double displacement_for_line_1 = 9.754314266668623e-01;
    // This quantity is how many pixels correspond to one Km/s 
    // of doppler shift of the line center for 6302.5017
    // (1/ 299792.458 * (6302.5017 * 1000)) / 21.549 =  9.755850745024434e-01;
    static const double displacement_for_line_2 = 9.755850745024434e-01;
    
    double center_displacement_in_pixels_line_1 =  displacement_for_line_1 * _par.lines[0].wavelength_shift.value;
    double center_displacement_in_pixels_line_2 =  displacement_for_line_2 * _par.lines[1].wavelength_shift.value;
    
    // A couple of checks to get sure this algorithm
    // does not blows
    if  (_par.n_regions != 2)
    {
        char nerror[32];
        sprintf(nerror, "%i", __LINE__);
        string t_error=string("\n") + __FILE__ + string(":") + string(nerror);
        t_error += ": The algorithm used to compute the magnitude of stokes V ";
        t_error += "expects two regions in the model. This model has only ";
        sprintf(nerror, "%i", _par.n_regions);
        t_error += string(nerror) + " regions. Please get sure there you either provide two regions ";
        t_error += "or you reconfigure this client to a different algorithm.";
        throw t_error;
    }
    if (_n_data != (_par.regions[0].n_wavelengths + _par.regions[1].n_wavelengths) )
    {
        char nerror[32];
        sprintf(nerror, "%i", __LINE__);
        string t_error=string("\n") + __FILE__ + string(":") + string(nerror);
        t_error += ": Internal trace error in the constrained vector. Please report to jgarcia@ucar.edu";
    }  
    
    // Now we find in which position the first center lies
    int index_for_center_1 = 1;
    double wavelength = _par.regions[0].first_wavelength;
    while ( fabs(wavelength - 6301.5091) > (.021549)) // Here we use
                                                      // the
                                                      // dispersion
                                                      // for hinode
    {
        index_for_center_1++;
        if (index_for_center_1 >  _par.regions[0].n_wavelengths)
        {
            char nerror[32];
            sprintf(nerror, "%i", __LINE__);
            string t_error=__FILE__ + string(":") + string(nerror);
            t_error += "All the region 1 has been scanned and the center was not found";
            throw t_error;
        } 
        wavelength += (.021549);
    }
    //cout<<"Found center for first line at index "<<index_for_center_1<<endl;
    index_for_center_1 = int(center_displacement_in_pixels_line_1) + index_for_center_1;
    //cout<<"This is the displaced center "<<index_for_center_1<<endl;
    
    // Now we find in which position the second center lies
    int index_for_center_2 = 1 + _par.regions[0].n_wavelengths;
    wavelength = _par.regions[1].first_wavelength;
    while ( fabs(wavelength - 6302.5017) > (.021549)) // Here we use
                                                      // the
                                                      // dispersion
                                                      // for hinode
    {
        index_for_center_2++;
        if (index_for_center_2 >  _n_data)
        {
            char nerror[32];
            sprintf(nerror, "%i", __LINE__);
            string t_error=__FILE__ + string(":") + string(nerror);
            t_error += "All the region 2 has been scanned and the center was not found";
            throw t_error;
        } 
        wavelength += (.021549);
    }
    //cout<<"Found center for second line at index "<<index_for_center_2<<endl;
    index_for_center_2 = int(center_displacement_in_pixels_line_1) + index_for_center_2;
    //cout<<"This is the displaced center "<<index_for_center_2<<endl;
    
    
    
    double signed_stokes_V = 0.0;
    register int yy;
    
    for (yy = 0; yy < index_for_center_1; yy++)
        signed_stokes_V += observed.V[yy];
    for (yy = index_for_center_1; yy < _par.regions[0].n_wavelengths; yy++)
        signed_stokes_V -= observed.V[yy];
    for (yy = _par.regions[0].n_wavelengths; yy < index_for_center_2; yy++)
        signed_stokes_V += observed.V[yy];
    for (yy =  index_for_center_2; yy < _n_data; yy++)
        signed_stokes_V -= observed.V[yy];
    
    
    return signed_stokes_V / _Imax;
}
