/* ============================================================================
 * Language: C++ 
 * File: MERLIN_MVoigt.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */


#include <vector>
using std::vector;

#include "MERLIN.h"
#include "MERLIN_Voigt.h"

#include "MERLIN_ZeemanComponent.h"
using CSAC::MERLIN_ZeemanComponent;


#include <iostream>
using std::cout;
using std::endl;


namespace CSAC
{
  
    void mvoigt(
                vector<MERLIN_ZeemanComponent> comp,
                double a,                     /* damping */
                double Delta_lambda,          /* displacement on Voigt */
                double H_doppler,             /* magnitude of the magnetic field divided by the width of the doppler line */
                double &Phi_p,              /* eta para luz circular derecha */
                double &Psi_p,              /* perfil anomalo */
                double &Phi_r,     /* eta para luz circular derecha */
                double &Psi_r,     /* perfil anomalo */
                double &Phi_b,      /* eta para luz circular izquierda */
                double &Psi_b       /* perfil anomalo */
                )
    {

	double xi;
	double DH, DF;
	
	Phi_p=0;
	Psi_p=0;
	Phi_r=0;
	Psi_r=0;
	Phi_b=0;
	Psi_b=0;
	
	unsigned int number_of_components=comp.size();
	
	for (int f=0; f < number_of_components; f++) {
		xi = Delta_lambda + comp[f].derivative * H_doppler;
		//cout<<"calling voigt for a= "<<a<<" and xi="<<xi<<endl;
		//call_voigt(&a, &xi, &DH, &DF, CSAC::VOIGT::FORTRAN);
		my_voigt(a,xi,DH,DF);
		switch (comp[f].type)
		{
			case MERLIN_ZeemanComponent::PI:
				{
					//cout<<"PI voigt values "<<DH<<" "<<DF<<endl;

					Phi_p = Phi_p + comp[f].strength * DH;
					Psi_p = Psi_p + comp[f].strength * DF;
				};
				break;
			case MERLIN_ZeemanComponent::SIGMA_RIGHT:
				{
					//cout<<"Sigma Right voigt values "<<DH<<" "<<DF<<endl;

					Phi_r = Phi_r + comp[f].strength * DH;
					Psi_r = Psi_r + comp[f].strength * DF;
				};
				break;
			case MERLIN_ZeemanComponent::SIGMA_LEFT:
				{
					//cout<<"Sigma Left voigt values "<<DH<<" "<<DF<<endl;

					Phi_b = Phi_b + comp[f].strength * DH;
					Psi_b = Psi_b + comp[f].strength * DF;
				};
				break;
		}
	}

	Psi_r = 2. * Psi_r;
	Psi_p = 2. * Psi_p;
	Psi_b = 2. * Psi_b;
	}

    void mvoigt_d( 
                  vector<MERLIN_ZeemanComponent> comp,
                  double a,                          /* damping */
                  double Delta_lambda,               /* displacement on Voigt */
                  double H_doppler,                  /* magnitude of the magnetic field divided by the width of the doppler line */
                  double &Phi_p,                     /* eta para luz circular derecha */
                  double &Psi_p,                     /* perfil anomalo */
                  double &Phi_r,                     /* eta para luz circular derecha */
                  double &Psi_r,                     /* perfil anomalo */
                  double &Phi_b,                     /* eta para luz circular izquierda */
                  double &Psi_b,                     /* perfil anomalo */
                  double dH_doppler__dB,
                  double dH_doppler__dDldop,
                  double dDelta_lambda__dDldop,
                  double dDelta_lambda__dv,
                  double & dPhi_p__dv,
                  double & dPhi_r__dv,
                  double & dPhi_b__dv,
                  double & dPhi_p__dB,
                  double & dPhi_r__dB,
                  double & dPhi_b__dB,
                  double & dPhi_p__da,
                  double & dPhi_r__da,
                  double & dPhi_b__da,
                  double & dPhi_p__dDldop,
                  double & dPhi_r__dDldop,
                  double & dPhi_b__dDldop,
                  double & dPsi_p__dB,
                  double & dPsi_b__dB,
                  double & dPsi_r__dB,
                  double & dPsi_p__dv,
                  double & dPsi_r__dv,
                  double & dPsi_b__dv,
                  double & dPsi_p__da,
                  double & dPsi_b__da,
                  double & dPsi_r__da,
                  double & dPsi_p__dDldop,
                  double & dPsi_b__dDldop,
                  double & dPsi_r__dDldop
                   )
    {
	double xi;
		
	double DH, DF;
	double dH__dXi, dF__dXi, dH__da, dF__da;
	
	Phi_p=0;
	Psi_p=0;
	Phi_r=0;
	Psi_r=0;
	Phi_b=0;
	Psi_b=0;
	dPhi_p__dv=0;
	dPhi_r__dv=0;
	dPhi_b__dv=0;
	dPhi_p__dB=0;
	dPhi_r__dB=0;
	dPhi_b__dB=0;
	dPhi_p__da=0;
	dPhi_r__da=0;
	dPhi_b__da=0;
	dPhi_p__dDldop=0;
	dPhi_r__dDldop=0;
	dPhi_b__dDldop=0;
	dPsi_p__dB=0;
	dPsi_b__dB=0;
	dPsi_r__dB=0;
	dPsi_p__dv=0;
	dPsi_r__dv=0;
	dPsi_b__dv=0;
	dPsi_p__da=0;
	dPsi_b__da=0;
	dPsi_r__da=0;
	dPsi_p__dDldop=0;
	dPsi_b__dDldop=0;
	dPsi_r__dDldop=0;
	
	unsigned int number_of_components=comp.size();
	
	for (int f=0; f<number_of_components; f++)
	{
		xi = Delta_lambda + comp[f].derivative * H_doppler;
		//cout<<"calling voigt for a= "<<a<<" and xi="<<xi<<endl;
		// call_voigt(&a, &xi, &DH, &DF, F);
		my_voigt(a,xi,DH,DF);

		switch (comp[f].type)
		{
			case MERLIN_ZeemanComponent::PI:
				{
					//cout<<"PI voigt values "<<DH<<" "<<DF<<endl;

					dH__dXi = -2 * xi * DH + 4 * a * DF;            /* derivative with respect of xi of the Voigt function   */
					dF__dXi = CSAC::Piis - a * DH - 2 * xi * DF;    /* derivative with respect of xi of the Faraday function */
					dF__da = dH__dXi / 2;                           /* derivative with respect of a of the Faraday function  */
					dH__da = -2 * dF__dXi;                          /* derivative with respect of a of the Voigt function    */

					Phi_p = Phi_p + comp[f].strength * DH;
					Psi_p = Psi_p + comp[f].strength * DF;

					dPhi_p__dB= dPhi_p__dB + comp[f].strength * dH__dXi * (comp[f].derivative * dH_doppler__dB);
					dPsi_p__dB= dPsi_p__dB + comp[f].strength * dF__dXi * (comp[f].derivative * dH_doppler__dB);

					dPhi_p__dDldop= dPhi_p__dDldop + comp[f].strength * dH__dXi * (comp[f].derivative * dH_doppler__dDldop + dDelta_lambda__dDldop);
					dPsi_p__dDldop= dPsi_p__dDldop + comp[f].strength * dF__dXi * (comp[f].derivative * dH_doppler__dDldop + dDelta_lambda__dDldop);

					dPhi_p__dv= dPhi_p__dv + comp[f].strength *  dH__dXi * dDelta_lambda__dv;
					dPsi_p__dv= dPsi_p__dv + comp[f].strength *  dF__dXi * dDelta_lambda__dv;

					dPhi_p__da= dPhi_p__da + comp[f].strength * dH__da;
					dPsi_p__da= dPsi_p__da + comp[f].strength * dF__da;
				};
				break;
			case MERLIN_ZeemanComponent::SIGMA_RIGHT:
				{
					//cout<<"Sigma Right voigt values "<<DH<<" "<<DF<<endl;

					dH__dXi = -2 * xi * DH + 4 * a * DF;            /* derivative with respect of xi of the Voigt function   */
					dF__dXi = CSAC::Piis - a * DH - 2 * xi * DF;    /* derivative with respect of xi of the Faraday function */
					dF__da = dH__dXi / 2;                           /* derivative with respect of a of the Faraday function  */
					dH__da = -2 * dF__dXi;                          /* derivative with respect of a of the Voigt function    */

					Phi_r = Phi_r + comp[f].strength * DH;
					Psi_r = Psi_r + comp[f].strength * DF;

					dPhi_r__dB= dPhi_r__dB + comp[f].strength * dH__dXi * (comp[f].derivative * dH_doppler__dB);
					dPsi_r__dB= dPsi_r__dB + comp[f].strength * dF__dXi * (comp[f].derivative * dH_doppler__dB);

					dPhi_r__dDldop= dPhi_r__dDldop + comp[f].strength * dH__dXi * (comp[f].derivative * dH_doppler__dDldop + dDelta_lambda__dDldop);
					dPsi_r__dDldop= dPsi_r__dDldop + comp[f].strength * dF__dXi * (comp[f].derivative * dH_doppler__dDldop + dDelta_lambda__dDldop);

					dPhi_r__dv= dPhi_r__dv + comp[f].strength *  dH__dXi * dDelta_lambda__dv;
					dPsi_r__dv= dPsi_r__dv + comp[f].strength *  dF__dXi * dDelta_lambda__dv;

					dPhi_r__da= dPhi_r__da + comp[f].strength * dH__da;
					dPsi_r__da= dPsi_r__da + comp[f].strength * dF__da;
				};
				break;
			case MERLIN_ZeemanComponent::SIGMA_LEFT:
				{
					//cout<<"Sigma Left voigt values "<<DH<<" "<<DF<<endl;

					dH__dXi = -2 * xi * DH + 4 * a * DF;            /* derivative with respect of xi of the Voigt function   */
					dF__dXi = CSAC::Piis - a * DH - 2 * xi * DF;    /* derivative with respect of xi of the Faraday function */
					dF__da = dH__dXi / 2;                           /* derivative with respect of a of the Faraday function  */
					dH__da = -2 * dF__dXi;                          /* derivative with respect of a of the Voigt function    */

					Phi_b = Phi_b + comp[f].strength * DH;
					Psi_b = Psi_b + comp[f].strength * DF;

					dPhi_b__dB= dPhi_b__dB + comp[f].strength * dH__dXi * (comp[f].derivative * dH_doppler__dB);
					dPsi_b__dB= dPsi_b__dB + comp[f].strength * dF__dXi * (comp[f].derivative * dH_doppler__dB);

					dPhi_b__dDldop= dPhi_b__dDldop + comp[f].strength * dH__dXi * (comp[f].derivative * dH_doppler__dDldop + dDelta_lambda__dDldop);
					dPsi_b__dDldop= dPsi_b__dDldop + comp[f].strength * dF__dXi * (comp[f].derivative * dH_doppler__dDldop + dDelta_lambda__dDldop);

					dPhi_b__dv= dPhi_b__dv + comp[f].strength *  dH__dXi * dDelta_lambda__dv;
					dPsi_b__dv= dPsi_b__dv + comp[f].strength *  dF__dXi * dDelta_lambda__dv;

					dPhi_b__da= dPhi_b__da + comp[f].strength * dH__da;
					dPsi_b__da= dPsi_b__da + comp[f].strength * dF__da;
				};
				break;
		}
	}
	
	Psi_r = 2. * Psi_r;
	Psi_p = 2. * Psi_p;
	Psi_b = 2. * Psi_b;
	dPsi_p__dB =  2. *dPsi_p__dB; 
	dPsi_r__dB =  2. *dPsi_r__dB; 
	dPsi_b__dB =  2. *dPsi_b__dB; 
	dPsi_p__dv =  2. *dPsi_p__dv;
	dPsi_r__dv =  2. *dPsi_r__dv;
	dPsi_b__dv =  2. *dPsi_b__dv;
	dPsi_p__da =  2. *dPsi_p__da;
	dPsi_r__da =  2. *dPsi_r__da;
	dPsi_b__da =  2. *dPsi_b__da;
	dPsi_p__dDldop =  2. *dPsi_p__dDldop;
	dPsi_r__dDldop =  2. *dPsi_r__dDldop;
	dPsi_b__dDldop =  2. *dPsi_b__dDldop;
    }

}; //end namespace CSAC
