/* ============================================================================
 * Language: C++ 
 * File: MERLIN_StrayLight.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#include "math.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

/*
 * Added to include the four1_FFTW() replacement call for the NR four1().
#include <fftw3.h>
*/

/*
 * Removed, contains NR code.

extern "C" {
    void realft(double data[], unsigned long n, int isign);
    void four1(double data[], unsigned long nn, int isign);
}
*/


/*
 * Added to include the four1_FFTW() replacement call for the NR four1().
 * and realft_FFTW replacement for NR realft()
*/
extern "C" {
    void four1_FFTW(double data[], unsigned long nn, int isign);
    void realft_FFTW(double data[], unsigned long n, int isign);
}


#include <iostream>
using std::cout;
using std::endl;

#include "MERLIN_StrayLight.h"
using CSAC::MERLIN_StrayLight;

/*
 * Added for the four1_FFTW() calls.
 */
/* extern "C" fftw_plan p1_FFTW; */
/* extern "C" fftw_plan p2_FFTW; */

// Copy constructor
MERLIN_StrayLight::MERLIN_StrayLight(const MERLIN_StrayLight &rhs) {
    _size = rhs._size;
    _continuum = rhs._continuum;
    _shifted_light = new double [_size];
    _derivative_shifted_light = new double [_size];
    _derivative_shifted_light_with_shift_zero  = new double [_size];
    _unshifted_light = new double [_size];
    _unshifted_light_in_fourier_space = new double [_size];
	
    // copy the data
    memcpy((void*) _unshifted_light, (void*) rhs._unshifted_light , (_size * sizeof(double)));	
    memcpy((void*) _shifted_light, (void*) rhs._shifted_light, (_size * sizeof(double)));  
    memcpy((void*) _derivative_shifted_light, (void*) rhs._derivative_shifted_light, (_size * sizeof(double)));  
    memcpy((void*) _derivative_shifted_light_with_shift_zero, (void*) rhs._derivative_shifted_light_with_shift_zero, (_size * sizeof(double)));	
    memcpy((void*) _unshifted_light_in_fourier_space, (void*) rhs._unshifted_light_in_fourier_space, (_size * sizeof(double)));	
    _shift = rhs._shift;	
    _ranges = rhs._ranges;	
    _set = rhs._set;	
}	

MERLIN_StrayLight::~MERLIN_StrayLight() {
    clean();
}

void MERLIN_StrayLight::clean() {
    if(_shifted_light) {
        delete [] _shifted_light;
        _shifted_light = 0;
    }
    if(_derivative_shifted_light) {
        delete [] _derivative_shifted_light;
        _derivative_shifted_light = 0;
    }
    if(_derivative_shifted_light_with_shift_zero) {
        delete [] _derivative_shifted_light_with_shift_zero;
        _derivative_shifted_light_with_shift_zero = 0;
    }
    if(_unshifted_light) {
        delete [] _unshifted_light;
        _unshifted_light = 0;	
    }
    if(_unshifted_light_in_fourier_space) {
        delete [] _unshifted_light_in_fourier_space;
        _unshifted_light_in_fourier_space = 0;
    }	
    _size = 0;	
    _set = false;
}

void MERLIN_StrayLight::set_stray_light(const vector<double> &l) throw(string) {
    // Lets get sure indeed the size of l is a power of two,
    // otherwise the call to realft will corrupt the memory.
    bool power_of_two = false;
    unsigned int pow = 2;
    while(pow <= l.size() ) {
        if(pow == l.size() ) {
            power_of_two = true;
            break;
        }
        else
            pow *= 2;
    }
    if(!power_of_two) {
        string error ("The provided vector to be loaded into a StrayLight object is not a power of two, ");
        error += string("you have provided me with  vector of size ");
        char my_buffer[48];
        snprintf(my_buffer, sizeof(my_buffer),"%i",l.size());
        error += my_buffer;
        error += string(".\nPlease check your stray light and if necessary pad it to the next current power of two which is size ");
        snprintf(my_buffer, sizeof(my_buffer),"%i",pow);
        error += my_buffer;
        throw error;
    }		
    clean();
    _size = l.size();
    _shifted_light = new double [_size];
    _derivative_shifted_light = new double [_size];
    _derivative_shifted_light_with_shift_zero = new double [_size];
    _unshifted_light = new double [_size];
    _unshifted_light_in_fourier_space = new double [_size];
		
    // copy the data given to the unshifted light buffer
    memcpy((void*) _unshifted_light, (void*) &l[0], (_size * sizeof(double)));
    
    // set the continuum for this data
    _continuum = l[0];
    for(int q=1; q<_size; q++) {
        if(l[q] > _continuum)  _continuum = l[q];
    }
	
    // since this is the initial set process the shifted light has a shift of zero
    // thus it is exactly the input data
    memcpy((void*) _shifted_light, _unshifted_light, (_size * sizeof(double) ) );
    _shift=0;
	
    // Now the buffer where the unshifted light in Fourier space will remain
    memcpy((void*) _unshifted_light_in_fourier_space, _unshifted_light, (_size * sizeof(double) ) );
	
    // Load the FFT of the unshifted light right in the same buffer
    // The realft function (as must Numerical Recipes (NR) routines) 
    // expect the buffer [1...n] and our buffer is indexed [0...n-1].
    ///We pass the buffer as expected by NR which is our
    // memory address before by substracting 1 from our pointer to double
    //realft(_unshifted_light_in_fourier_space-1, _size, 1);
    
    // Changed to use FFTW instead of Numerical Recipes realft.
	realft_FFTW(_unshifted_light_in_fourier_space-1, _size, 1);
    // fftw_destroy_plan(p1_FFTW);
    // fftw_destroy_plan(p2_FFTW);
  
    int full_size = _size * 2;
    double *my_shifted_light;
    my_shifted_light = new double [full_size];
  
    // organized the shifted light in Fourier alternating real an imaginary parts
    // First element
    my_shifted_light[0] = _unshifted_light_in_fourier_space[0];
    my_shifted_light[1] = 0;
    
    for(register int i = 2; i < _size; i += 2) {
        // real part
        my_shifted_light[i]   = _unshifted_light_in_fourier_space[i]; 
        // imaginary part
        my_shifted_light[i+1] = _unshifted_light_in_fourier_space[i+1];
    }
    
    // Nyquist frequency
    my_shifted_light[_size] = _unshifted_light_in_fourier_space[1];
    my_shifted_light[_size+1] = 0;
    
    // Index for the derivative factor
    int index = 0;
    // derivative factor, this number is purely imaginary.
    double derivative_factor;
    // Space to hold the complex multiplication results
    double real_element;
    double imaginary_element;
    // In the loop notice that because we have <= _size
    // effectively we are including the Nyquist frequency. 
    // By multiplying times the derivative factor we get the 
    // derivative with respect of the current shift
    // In this routine since we are just setting the stray light
    // the shift is zero.
    for (int i = 0; i <= _size; i += 2) {
        derivative_factor = 2 * CSAC::Pi * index / _size;
        real_element = - my_shifted_light[i+1] * derivative_factor; // Imaginary times Imaginary with the - sign due to (SQRT(-1))^2
        imaginary_element = my_shifted_light[i] * derivative_factor; // Real times Imaginary
        my_shifted_light[i] = real_element;
        my_shifted_light[i+1] = imaginary_element;
        index++;
    }
    // Now the symmetric part, using the complex conjugate
    // After we are done with this loop, the derivative
    // is contained inside the buffer my_shifted_light.
    int j = _size - 2;
    for(register int i = _size + 2; i < full_size; i += 2) {
        my_shifted_light[i] = my_shifted_light[j]; 
        my_shifted_light[i+1] = - my_shifted_light[j+1];
        j -= 2;
    }
    // Get the derivative for the signal shifted in real space
    four1_FFTW(my_shifted_light - 1, _size, -1);
    // fftw_destroy_plan(p1_FFTW);
    // fftw_destroy_plan(p2_FFTW);
    j=0;
    // load in the internal buffer just the real part
    // Notice how the value is corrected dividing by _size
    // This is explained in the four1 routine for Numerical
    // recipes in C.
    for(int i =0; i < full_size; i +=2) {
        _derivative_shifted_light[j] = my_shifted_light[i]/_size; 
        j++;
    }
    
    // Now this is _derivative_shifted_light is all set
    // however we make a copy of it in case that in the future
    // we are requested a shift of zero, thus we avoid recalculating
    // the derivative wrt shift = zero
    memcpy((void*) _derivative_shifted_light_with_shift_zero, _derivative_shifted_light, (_size * sizeof(double) ) );
    
    // Now the object is set and ready to go
    _set=true;
    
    delete [] my_shifted_light;
}

void MERLIN_StrayLight::set_ranges (const vector<CSAC::MERLIN_Range> &ranges) throw (string) {
    _ranges.clear();
    int last_index = 0;
    for(register int i=0; i < ranges.size(); i++) {
        if(ranges[i].last < ranges[i].first) {
            string error ("The provided range indexes are wrong, operating on range ");
            char my_buffer[250];
            snprintf(my_buffer, sizeof(my_buffer),"%i",i);
            error += my_buffer;
            error += string(" the last value is set to ");
            snprintf(my_buffer, sizeof(my_buffer),"%i",ranges[i].last);
            error += my_buffer;
            error += string(" while the first value is set to ");
            snprintf(my_buffer, sizeof(my_buffer),"%i",ranges[i].first);
            error += my_buffer;
            throw error;
        }
        if (ranges[i].first < 0)
            throw string("The provided range has the first index negative");
        if (last_index < ranges[i].last)
            last_index = ranges[i].last;
        if (last_index > (_size -1) )
            throw string("The provided range last index will cause a buffer overflow in the MERLIN_StrayLight data section");
        _ranges.push_back(ranges[i]);
    }
}

void MERLIN_StrayLight::shift_light(const double &xshift) throw(string) {
    if(!set()) throw string("The current object is not set and you are trying to shift it");		
    if(xshift == _shift) return;	
    _shift= xshift;	
    if(_shift == 0) {
        // simple copy the buffer that we have of the unshifted light
        memcpy((void*) _shifted_light, _unshifted_light, (_size * sizeof(double)));
        memcpy((void*) _derivative_shifted_light, _derivative_shifted_light_with_shift_zero, (_size * sizeof(double) ) );
        return;
    }
	
    // We have to build the phase for the shift and execute the shift in Fourier space
    double real_element_of_phase;
    double imaginary_element_of_phase;
  
    // This buffer is local to hold the shifted light.
    // Notice that althought we original signal is real
    // and that we just just need 1/2 of the space to hold
    // this data in Fourier space, the shift will make the 
    // signal in Fourier space not to be exactly the result
    // of real input data. If the shift is integer, the aliasing
    // effects do not matter, however, when the shift is real
    // aliasing effects must be solved by "choosing" an interpolant.
    // In our case, we chose to make the signal symmetric with respect
    // to the Nyquist frequency as it is was bandlimited.
    // Data is holded in the following order:
    // my_shifted_light[0] = First complex, real part
    // my_shifted_light[1] = First complex, imaginary part
    // and so on...
    int full_size = _size * 2;
	double *my_shifted_light;
    my_shifted_light = new double [full_size];
  
    // Index for the phase factor
    int index = 0;
  
    // First element ...
    real_element_of_phase = cos(2 * CSAC::Pi * index * _shift / _size);
    imaginary_element_of_phase = sin(2 * CSAC::Pi * index * _shift / _size);
    my_shifted_light[0] = _unshifted_light_in_fourier_space[0] * real_element_of_phase;
    my_shifted_light[1] = _unshifted_light_in_fourier_space[0] * imaginary_element_of_phase;
    index++;
    
    for(register int i = 2; i < _size; i += 2) {
        real_element_of_phase = cos(2 * CSAC::Pi * index * _shift / _size);
        imaginary_element_of_phase = sin(2 * CSAC::Pi * index * _shift / _size);
        // real part
        my_shifted_light[i]   = _unshifted_light_in_fourier_space[i] * real_element_of_phase      - _unshifted_light_in_fourier_space[i+1] * imaginary_element_of_phase;
        // imaginary part
        my_shifted_light[i+1] = _unshifted_light_in_fourier_space[i] * imaginary_element_of_phase + _unshifted_light_in_fourier_space[i+1] * real_element_of_phase;	
        index++;
        //cout<<"I = "<<i<<"; Index = "<<index<<endl;
    }
    
    // Nyquist frequency
    real_element_of_phase = cos(2 * CSAC::Pi * index * _shift / _size);
    imaginary_element_of_phase = sin(2 * CSAC::Pi * index * _shift / _size);
    my_shifted_light[_size] = _unshifted_light_in_fourier_space[1] * real_element_of_phase;
    my_shifted_light[_size+1] = _unshifted_light_in_fourier_space[1] * imaginary_element_of_phase;
    //cout<<"Index now is "<<index<<endl<<endl<<endl;
    // Now the symmetric part, using the complex conjugate
    int j = _size - 2;
    for(register int i = _size + 2; i < full_size; i += 2) {
        my_shifted_light[i] = my_shifted_light[j]; 
        my_shifted_light[i+1] = - my_shifted_light[j+1];
        j -= 2;
    }
    // get a copy for the derivative with respect of the shift
    // before the shifted light buffer changes in the four1 call
	double *copy_my_shifted_light;
    copy_my_shifted_light = new double [full_size];
    memcpy(copy_my_shifted_light, my_shifted_light, (full_size * sizeof(double)));
    // Get the signal shifted in real space
    four1_FFTW(my_shifted_light - 1, _size, -1);
    // fftw_destroy_plan(p1_FFTW);
    // fftw_destroy_plan(p2_FFTW);
    j=0;
    // load in the internal buffer just the real part
    for(int i =0; i < full_size; i +=2) {
        _shifted_light[j] = my_shifted_light[i]/_size; 
        //cout<<"----> "<<_shifted_light[j]<<endl;
        j++;
    }
    index = 0;
    // derivative factor, this number is purely imaginary.
    double derivative_factor;
    // Space to hold the complex multiplication results
    double real_element;
    double imaginary_element;
    // In the loop notice that because we have <= _size
    // effectively we are including the Nyquist frequency. 
    for(int i = 0; i <= _size; i += 2) {
        derivative_factor = 2 * CSAC::Pi * index / _size;
        real_element = - copy_my_shifted_light[i+1] * derivative_factor; // Imaginary times Imaginary with the - sign due to (SQRT(-1))^2
        imaginary_element = copy_my_shifted_light[i] * derivative_factor; // Imaginary times real
        copy_my_shifted_light[i] = real_element;
        copy_my_shifted_light[i+1] = imaginary_element;
        index++;
    }
    // Now the symmetric part, using the complex conjugate
    j = _size - 2;
    for(register int i = _size + 2; i < full_size; i += 2) {
        copy_my_shifted_light[i] = copy_my_shifted_light[j]; 
        copy_my_shifted_light[i+1] = - copy_my_shifted_light[j+1];
        j -= 2;
    }
    // Get the derivative for the signal shifted in real space
    four1_FFTW(copy_my_shifted_light - 1, _size, -1);
    // fftw_destroy_plan(p1_FFTW);
    // fftw_destroy_plan(p2_FFTW);
    j=0;
    // load in the internal buffer just the real part
    for(int i =0; i < full_size; i +=2) {
        _derivative_shifted_light[j] = copy_my_shifted_light[i]/_size; 
        //cout<<"@@@@@@@@@ "<<copy_my_shifted_light[i]<<endl;
        j++;
    }
    delete [] my_shifted_light;
    delete [] copy_my_shifted_light;
}

void MERLIN_StrayLight::derivative_wrt_current_shift(vector<double> &data) throw(string) {
    if(!set()) throw string("The current object is not set and you are trying to shift it");		
    data.resize(_size,0.0);
    for(int i=0; i < _size; i++) data[i] = _derivative_shifted_light[i];
}

double& MERLIN_StrayLight::operator[](int i) throw (string) {
    if( (0<=i) && (i<_size) ) {
        return _shifted_light[i];
    }
    else 
        throw string("MERLIN_StrayLight object out of bounds");
}

double& MERLIN_StrayLight::operator[](int i) const throw (string) {
    if( (0<=i) && (i<_size) ) {
        return _shifted_light[i];
    }
    else
        throw string("MERLIN_StrayLight object out of bounds");
}

void MERLIN_StrayLight::load_constrained_stray_light_vector(vector<double> &csl) {
    int n_data = 0;	
    for(int i=0; i < _ranges.size(); i++) 
        n_data += (_ranges[i].last - _ranges[i].first + 1);	
    csl.resize(n_data, 0.0);
    int h = 0;	
    for(int i=0; i < _ranges.size(); i++) {
        for(int j=_ranges[i].first; j <=_ranges[i].last; j++)  
            csl[h++] = _shifted_light[j];
    }	
}

void MERLIN_StrayLight::load_constrained_derivative_wrt_current_shift(vector<double> &csl) {
    int n_data = 0;	
    for(int i=0; i < _ranges.size(); i++) 
        n_data += (_ranges[i].last - _ranges[i].first + 1);	
    csl.resize(n_data, 0.0);
    int h = 0;	
    for(int i=0; i < _ranges.size(); i++) {
        for(int j=_ranges[i].first; j <=_ranges[i].last; j++)  
            csl[h++] = _derivative_shifted_light[j];
    }
}

void MERLIN_StrayLight::load_constrained_stray_light_vector_wavelength(vector<double> &wl) {
    int n_data = 0;	
    for(int i=0; i < _ranges.size(); i++) 
        n_data += (_ranges[i].last - _ranges[i].first + 1);	
    wl.resize(n_data, 0.0);
    int h = 0;	
    for(int i=0; i < _ranges.size(); i++) {
        for(int j=_ranges[i].first; j <=_ranges[i].last; j++)  
            wl[h++] = _ranges[i].wave_base + ( (j - _ranges[i].first) * _ranges[i].wave_step ) / 1000;
    }
}

MERLIN_StrayLight& 
MERLIN_StrayLight::operator=(const MERLIN_StrayLight &rhs) {
    if(this == &rhs)
        return *this;  
    if(_size != rhs._size) {
        clean();
        _size = rhs._size;
        _shifted_light = new double [_size];
        _derivative_shifted_light = new double [_size];
        _derivative_shifted_light_with_shift_zero = new double [_size];
        _unshifted_light = new double [_size];
        _unshifted_light_in_fourier_space = new double [_size];
    }	
    // copy the data
    _continuum = rhs._continuum;    
    memcpy((void*) _unshifted_light, (void*) rhs._unshifted_light , (_size * sizeof(double)));	
    memcpy((void*) _shifted_light, (void*) rhs._shifted_light, (_size * sizeof(double)));  
    memcpy((void*) _derivative_shifted_light, (void*) rhs._derivative_shifted_light, (_size * sizeof(double)));  
    memcpy((void*) _derivative_shifted_light_with_shift_zero, (void*) rhs._derivative_shifted_light_with_shift_zero, (_size * sizeof(double)));	
    memcpy((void*) _unshifted_light_in_fourier_space, (void*) rhs._unshifted_light_in_fourier_space, (_size * sizeof(double)));	
    _shift = rhs._shift;	
    _ranges = rhs._ranges;	
    _set = rhs._set;	
    return *this;
}

void MERLIN_StrayLight::dump( ostream &strm ) const {
    strm.precision(15);
    strm<<"[";
    for(int i=0; i < _size; i++) {
        strm<<_unshifted_light[i];
        if (i < (_size -1))
            strm<<", ";
    }
    strm<<"]"<<endl;  
    strm<<"The following ranges are loaded: "<<endl;
    for(int i=0; i < _ranges.size(); i++) {
        strm<<"Range "<<i+1
        <<": First = "<<_ranges[i].first
        <<" Last = "<<_ranges[i].last
        <<" wave_base = "<<_ranges[i].wave_base
        <<" wave_step = "<<_ranges[i].wave_step<<endl;
    }        
}
