/* ============================================================================
 * Language: C++ 
 * File: MERLIN_Voigt.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */



/********************************************************************************
 *             National Center for Atmospheric Research (NCAR)                  *
 *             High Altitude Observatory (HAO)                                  * 
 *             Boulder, CO.                                                     * 
 *             Author: Jose H. Garcia.                                          *
 *             Language: C                                                      *
 *             Based on the work of Hector Socas Navarro.                       *
 *             Original comments included below.                                *                
 *             Description. This routine calculates the Voigt function.         *
 *             Input Parameters: damping, frequency offset.                     *
 *             Output Parameters: voigt (absortion), faraday(dispersion)        *
 *             For full documentation please refer to the companion             *
 *             TeX file voigt.tex                                               *   
 ********************************************************************************  
 * This vectorizable Voigt function is based on the paper by Hui,               *  
 * Armstrong and Wray, JQSRT 19, 509 (1977). Errors become                      *  
 * significant (at the < 1% level) around the knee between the                  *  
 * Doppler core and the damping wings for 0.0 < a < 0.001. The                  *  
 * normalization is such that the integral is equal to SQRT(PI).                *  
 * If J <> 0 this function returns the dispersion function.                     *  
 * --------------------------------------------------------------               *  
 * Authors: Jack Harvey, Aake Nordlund.                                         *  
 * Modified by Sami Solanki (1985).                                             *  
 * Modified by A.D. Wittmann (1986) to include F(a,-v) and F(0,v)               *
 * convertida en rutina por Basilio Ruiz (1993)                                 *
 * --------------------------------------------------------------               *  
 *  Last Update: 22-jun 93.                                                     *  
 *                                                                              *
 ********************************************************************************/

#include <math.h>
#include <complex>
#include <iostream>
#include <string>
#include <stdio.h>

#include "MERLIN_Voigt.h"

#include "config.h"

#define VOIGT2_F77 F77_FUNC (voigt2, VOIGT2)

// This is the function coming from voigt2.f
extern "C" 
{
    void VOIGT2_F77 (double *, double*, double*, double *);
}

using std::complex ;
using std::cerr;
using std::endl;
using std::string;

void voigt_cplusplus (
                      double damping, 
                      double frequency_offset, 
                      double *voigt, 
                      double *faraday
                      )
{
  
    /* coefficients of the rational approximation formula
       to the complementary error function */ 
    static const double A0 = 122.607931777104326;
    static const double A1 = 214.382388694706425;
    static const double A2 = 181.928533092181549;
    static const double A3 = 93.155580458138441;
    static const double A4 = 30.180142196210589;
    static const double A5 = 5.912626209773153;
    static const double A6 = 0.564189583562615;
    static const double B0 = 122.60793177387535;
    static const double B1 = 352.730625110963558;  
    static const double B2 = 457.334478783897737;
    static const double B3 = 348.703917719495792;
    static const double B4 = 170.354001821091472;
    static const double B5 = 53.992906912940207;
    static const double B6 = 10.479857114260399;
    static const double XDWS[]={
        .1,.2,.3,.4,.5,.6,.7,.8,.9,1.,1.2,1.4,1.6,1.8,2.,   
        3.,4.,5.,6.,7.,8.,9.,10.,12.,14.,16.,18.,20.
    };
    static const double YDWS[]={   
        9.9335991E-02,1.9475104E-01,2.8263167E-01,3.5994348E-01,  
        4.2443639E-01,4.7476321E-01,5.1050407E-01,5.3210169E-01,  
        5.4072434E-01,5.3807950E-01,5.0727350E-01,4.5650724E-01,  
        3.9993989E-01,3.4677279E-01,3.0134040E-01,1.7827103E-01,  
        1.2934799E-01,1.0213407E-01,8.4542692E-02,7.2180972E-02,  
        6.3000202E-02,5.5905048E-02,5.0253846E-02,4.1812878E-02,  
        3.5806101E-02,3.1311397E-02,2.7820844E-02,2.5031367E-02
    };
  
    int i, k, kk, kkk;
    int ivsigno;
    double V;
    double Y;
    double D, D1, D2, D3, D12, D13, D23;
    complex<double> Z; 
  
    if(frequency_offset<0)
        ivsigno=-1;
    else
        ivsigno=1;
  
    V=ivsigno*frequency_offset;
  
    /* If the damping is zero, then the 
       convolution is dominated by the gaussian 
    */
    if (damping==0)
    {
        *voigt=exp(-(V*V));
    
        if (V>XDWS[0])
        {
            if (V>XDWS[27])
	    {
                Y=0.5/V;
                D=Y*(1+Y/V);
	    }
            else
	    {
                k=26;
                i=1;
                while(i<27)
                {
                    if(XDWS[i]<V)
                        i++;
                    else
                    {
                        k=i;
                        break;
                    }
                }
                kk=k-1;
                kkk=k+1;
                D1=V-XDWS[kk]; 
                D2=V-XDWS[k];  
                D3=V-XDWS[kkk];
                D12=XDWS[kk]-XDWS[k];  
                D13=XDWS[kk]-XDWS[kkk];
                D23=XDWS[k]-XDWS[kkk];
                D=YDWS[kk]*D2*D3/(D12*D13)-YDWS[k]*D1*D3/(D12*D23)+YDWS[k]*D1*D2/(D13*D23);
	    }
        }
        else
            D=V*(1.-.66666667*(V*V));
        *faraday=D*ivsigno*5.641895836E-1;
    }
    else
    {
        Z=complex<double>(damping, -V) ; 
        Z=((((((A6*Z+A5)*Z+A4)*Z+A3)*Z+A2)*Z+A1)*Z+A0)/   
        (((((((Z+B6)*Z+B5)*Z+B4)*Z+B3)*Z+B2)*Z+B1)*Z+B0);  
        *voigt=Z.real();
        *faraday=.5*ivsigno*Z.imag();
    }
}

namespace CSAC
{
    namespace VOIGT
    {
    
        void call_voigt(
                        double *damping, 
                        double *frequency_offset, 
                        double *voigt, 
                        double *faraday,
                        enum CSAC::VOIGT::v_type type
                        )
        {
            if (type == CSAC::VOIGT::FORTRAN)
                VOIGT2_F77(damping, frequency_offset, voigt, faraday);
            else if (type == CSAC::VOIGT::CPLUSPLUS)
                voigt_cplusplus(*damping, *frequency_offset, voigt, faraday);
            else
            {
                char error_line[32];
                sprintf(error_line, "%i", __LINE__);
                string t_error=__FILE__ + string(":") + string(error_line) + ": Invalid type for Voigt function";
                throw t_error; 
            }
        }
    }
}

namespace CSAC
{
    namespace VOIGT
    {
        /* coefficients of the rational approximation formula
           to the complementary error function 
        */ 
        const double A0 = 122.607931777104326;
        const double A1 = 214.382388694706425;
        const double A2 = 181.928533092181549;
        const double A3 = 93.155580458138441;
        const double A4 = 30.180142196210589;
        const double A5 = 5.912626209773153;
        const double A6 = 0.564189583562615;
        const double B0 = 122.60793177387535;
        const double B1 = 352.730625110963558;  
        const double B2 = 457.334478783897737;
        const double B3 = 348.703917719495792;
        const double B4 = 170.354001821091472;
        const double B5 = 53.992906912940207;
        const double B6 = 10.479857114260399;
        const double XDWS[]={
            .1,.2,.3,.4,.5,.6,.7,.8,.9,1.,1.2,1.4,1.6,1.8,2.,   
            3.,4.,5.,6.,7.,8.,9.,10.,12.,14.,16.,18.,20.
        };
        const double YDWS[]={   
            9.9335991E-02,1.9475104E-01,2.8263167E-01,3.5994348E-01,  
            4.2443639E-01,4.7476321E-01,5.1050407E-01,5.3210169E-01,  
            5.4072434E-01,5.3807950E-01,5.0727350E-01,4.5650724E-01,  
            3.9993989E-01,3.4677279E-01,3.0134040E-01,1.7827103E-01,  
            1.2934799E-01,1.0213407E-01,8.4542692E-02,7.2180972E-02,  
            6.3000202E-02,5.5905048E-02,5.0253846E-02,4.1812878E-02,  
            3.5806101E-02,3.1311397E-02,2.7820844E-02,2.5031367E-02
        };
    
        int i;
        int k;
        int kk; 
        int kkk;
        int ivsigno;
        double V;
        double Y;
        double D;
        double D1;
        double D2;
        double D3;
        double D12; 
        double D13; 
        double D23;
        double Z1_real ;
        double Z1_imag;
        double Z2_real;
        double Z2_imag;
        double Z3_real;
        double Z3_imag;
        double Z4_real;
        double Z4_imag;
        double Z5_real;
        double Z5_imag;
        double Z6_real;
        double Z6_imag;
        double ZZ1_real;
        double ZZ1_imag;
        double ZZ2_real;
        double ZZ2_imag;
        double ZZ3_real;
        double ZZ3_imag;
        double ZZ4_real;
        double ZZ4_imag;
        double ZZ5_real;
        double ZZ5_imag;
        double ZZ6_real;
        double ZZ6_imag;
        double ZZ7_real;
        double ZZ7_imag;
        double division_factor;
        double ZZZ_real;
        double ZZZ_imag;
    
    }; // end namespace CSAC::voigt
} // end namespace CSAC

